#ifndef BASESUBS_HPP_SENTRY
#define BASESUBS_HPP_SENTRY

#include <scriptpp/scrmacro.hpp>


class IfForm : public ScriptMacroprocessor::Macro {
public:
    IfForm() : ScriptMacroprocessor::Macro("if") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class OrForm : public ScriptMacroprocessor::Macro {
public:
    OrForm() : ScriptMacroprocessor::Macro("or") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class IfEqForm : public ScriptMacroprocessor::Macro {
public:
    IfEqForm() : ScriptMacroprocessor::Macro("ifeq") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class IfBelongs : public ScriptMacroprocessor::Macro {
public:
    IfBelongs() : ScriptMacroprocessor::Macro("ifbelongs") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class SwitchForm : public ScriptMacroprocessor::Macro {
public:
    SwitchForm() : ScriptMacroprocessor::Macro("switch") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};


    // NB: false by default, this property is used!
class IfCond : public ScriptMacroprocessor::Macro {
    bool cond;
public:
    IfCond(const char *name)
        : ScriptMacroprocessor::Macro(name), cond(false) {}
    ScriptVariable Expand(const ScriptVector &params) const;
    void SetCond(bool c) { cond = c; }
};

class ForEach : public ScriptMacroprocessor::Macro {
    ScriptMacroprocessor *the_master;
public:
    ForEach(ScriptMacroprocessor *m)
        : ScriptMacroprocessor::Macro("foreach"), the_master(m) {}
    ScriptVariable Expand(const ScriptVector &params) const;
};


class ImgFileDimensions : public ScriptMacroprocessor::Macro {
    ScriptVariable basepath;
public:
    ImgFileDimensions(const ScriptVariable &bp)
        : ScriptMacroprocessor::Macro("imgdim"), basepath(bp)
    {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class IfFile : public ScriptMacroprocessor::Macro {
    ScriptVariable basepath;
public:
    IfFile(const ScriptVariable &bp)
        : ScriptMacroprocessor::Macro("iffile"), basepath(bp) {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class FileSize : public ScriptMacroprocessor::Macro {
    ScriptVariable basepath;
public:
    FileSize(const ScriptVariable &bp)
        : ScriptMacroprocessor::Macro("filesize"), basepath(bp) {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class ReadFile : public ScriptMacroprocessor::Macro {
    ScriptVariable basepath;
public:
    ReadFile(const ScriptVariable &bp)
        : ScriptMacroprocessor::Macro("readfile"), basepath(bp) {}
    ScriptVariable Expand(const ScriptVector &params) const;
};



class AttributeQuote : public ScriptMacroprocessor::Macro {
public:
    AttributeQuote() : ScriptMacroprocessor::Macro("q") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class RemoveLF : public ScriptMacroprocessor::Macro {
public:
    RemoveLF() : ScriptMacroprocessor::Macro("rmlf") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class CollapseWS : public ScriptMacroprocessor::Macro {
public:
    CollapseWS() : ScriptMacroprocessor::Macro("collapsews") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class Trim : public ScriptMacroprocessor::Macro {
public:
    Trim() : ScriptMacroprocessor::Macro("trim") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class SubstitutionURLEnc : public ScriptMacroprocessor::Macro {
public:
    SubstitutionURLEnc() : ScriptMacroprocessor::Macro("urlenc") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};



class MacroRFC2822Date : public ScriptMacroprocessor::Macro {
public:
    MacroRFC2822Date() : ScriptMacroprocessor::Macro("rfcdate") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};

class NowTime : public ScriptMacroprocessor::Macro {
public:
    NowTime() : ScriptMacroprocessor::Macro("now") {}
    ScriptVariable Expand(const ScriptVector &params) const;
};


#endif
